@@ -34,7 +34,7 @@ struct Queue {
 
     /// This function does not check if there is enough space in the circular buffer, assuming the check
     /// has been performed.
-    void circular_buffer_write(uint8_t *buffer, uint8_t *data, size_t data_size) {
+    void circular_buffer_write(uint8_t *buffer, const uint8_t *data, const size_t data_size) {
         if (tail + data_size < max_size_bytes) {
             // all data fits before the wrapping point
             memcpy(buffer + tail, data, data_size);
@@ -109,7 +109,7 @@ size_t queue_object_size() {
 }
 
 void create_queue(void *queue_obj_memory, size_t max_size_bytes) {
-    const auto q = new(queue_obj_memory) Queue(max_size_bytes);
+    new(queue_obj_memory) Queue(max_size_bytes);  // placement new
 }
 
 struct timeval float_seconds_to_timeval(float seconds) {
@@ -145,28 +145,37 @@ bool timer_positive(const struct timeval &timer) {
     return (timer.tv_sec > 0) || (timer.tv_sec == 0 && timer.tv_usec > 0);
 }
 
-int queue_put(void *queue_obj, void *buffer, void *msg_data, size_t msg_size, int block, float timeout) {
+int queue_put(void *queue_obj, void *buffer, const void **msgs_data, const size_t *msg_sizes, const size_t num_msgs, const int block, const float timeout) {
     auto q = (Queue *)queue_obj;
     LockGuard lock(&q->mutex);
 
-    auto wait_remaining = float_seconds_to_timeval(timeout);
-    while (!q->can_fit(msg_size + sizeof(msg_size))) {
-        if (!block || !timer_positive(wait_remaining))
-            return Q_FULL;
+    {
+        size_t total_size = num_msgs * sizeof(size_t);
+        for (size_t i = 0; i < num_msgs; ++i)
+            total_size += msg_sizes[i];
 
-        wait_remaining = wait(wait_remaining, &q->not_full, &q->mutex);
+        auto wait_remaining = float_seconds_to_timeval(timeout);
+        while (!q->can_fit(total_size)) {
+            if (!block || !timer_positive(wait_remaining))
+                return Q_FULL;
+
+            wait_remaining = wait(wait_remaining, &q->not_full, &q->mutex);
+        }
     }
 
-    // write the size of the message to the circular buffer
-    q->circular_buffer_write((uint8_t *)buffer, (uint8_t *)&msg_size, sizeof(msg_size));
+    for (size_t i = 0; i < num_msgs; ++i) {
+        // write the size to the circular buffer
+        q->circular_buffer_write((uint8_t *)buffer, reinterpret_cast<const uint8_t *>(msg_sizes + i), sizeof(size_t));
 
-    // write the message itself
-    q->circular_buffer_write((uint8_t *)buffer, (uint8_t *)msg_data, msg_size);
+        // write the message to the circular buffer
+        q->circular_buffer_write((uint8_t *)buffer, reinterpret_cast<const uint8_t *>(msgs_data[i]), msg_sizes[i]);
+
+        // Increment count by one as one element has been added
+        ++q->num_elem;
+    }
 
     pthread_cond_signal(&q->not_empty);
-    
-    // Increment count by one as one element has been added
-    ++q->num_elem;
+
     return Q_SUCCESS;
 }
 
